/*
** graburl.c - part of GrabURL 2.0
** Copyright (C) 1996-97 Serge Emond
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "graburl.h"
#include "config.h"
#include "log.h"
#include "http.h"
#include "scan.h"
#include "libgurl.h"
#include "macros.h"

char *gu_version = GU_VERSION;

struct Grab *grab;

int loop(void);

int main(int argc, char **argv)
{
  int i;

  InitConfig();
  i = GetConfigArg(argv);
  if (i > 0)
	ReadConfigFile(argv[i]);
  else {
    /* No given config file - try in user's home */
    char *home;
    
    home = getenv("HOME");
    if (home) {
      char *file;
      
      file = (char *)gmalloc(strlen(home) + 12);
      strcpy(file, home);
      strcat(file, "/.graburlrc");
      if (access(file, R_OK) == 0) {
	ReadConfigFile(file);
	i = 1;
      }
      free(file);
    }
    if (!(i > 0)) {
      /* We try config in /etc */
#ifdef __SASC
      if (access("etc:graburlrc", R_OK) == 0) {
	ReadConfigFile("etc:graburlrc");
#else
      if (access("/etc/graburlrc", R_OK) == 0) {
	ReadConfigFile("/etc/graburlrc");
#endif
      }
      else {
	fprintf(stderr, "graburl: Error, no config file\n");
	exit(EXIT_FAILURE);
      }
    }
  }
  
  GetArgs(argv);
  
  uri_list.first = 0;
  uri_list.last = 0;
  PostConfig();
  
  if (access(cfg_global.saveroot, R_OK | W_OK | X_OK)) {
    fprintf(stderr, "Error, can't access SaveRoot \"%s\"\n",
	    cfg_global.saveroot);
    exit(EXIT_FAILURE);
  }
  
  if ((cfg_global.workfile) && (access(cfg_global.workfile, R_OK) == 0))
    LoadURIFile(cfg_global.workfile);
  
  pdebug("Debug is *ON*");
  
  /* We call the loop */
  loop();
  
  exit(EXIT_SUCCESS);
}

/*
** The loop:
** - check for url, and get it
** - save workfile if any
*/

int current;

int loop()
{
  struct URI *curi;
  int is_retrying, ret;
  struct timeval tv;
  char *cp;
  
  is_retrying = 0; current = 0;
  
  while (1) {
    curi = FindURI(0, 0,
		   URI_SKIP | URI_ERROR | URI_MOVED | URI_RECEIVED |
		   URI_FAILED);
    if (!curi) {
      break;
    }
    
    if (current && (cfg_global.wait.tv_sec || cfg_global.wait.tv_usec)) {
      fprintf(stderr, "Delaying...\033[A\n");
      tv.tv_sec = cfg_global.wait.tv_sec;
      tv.tv_usec = cfg_global.wait.tv_usec;
      select(0,0,0,0,&tv);
      fprintf(stderr, "\033[K\033[A\n");
    }
    ret = GetHTTPFile(grab, curi, 0);
    
    if (ret) {
      current++;
      if (grab->uri->flags & URI_SCAN) {
	cp = HTTP_FindHeadLine("Content-Type:");
	if ((cp && !str_nieq(cp+14, "text/html", 9)) ||
	    stristr(grab->filename, ".htm")) {
	  ScanFile(grab);
	}
      }
    }
    
    HTTP_PostGetFile(grab);
    if (!curi->saveto) free(grab->filename);
    
    if (cfg_global.workfile) SaveURIFile(cfg_global.workfile);
    
  }
  return(0);
}
